@@ -1,40 +1,169 @@
-package autoupdate;  # $Date: 2002/04/12 09:20:23 $
+package autoupdate;  # $Date: 2002/10/12 $
 
 use strict;
 # warnings causes problems under 5.005
 #use warnings;
 use POSIX;
-#use RPM::Header;
 
 BEGIN {
 	use Exporter   ();
 	use vars qw($VERSION $DEBUG @ISA @EXPORT @EXPORT_OK);
 
 	# set the version for version checking
-	$VERSION     = 1.20;
+	$VERSION     = 1.4.1;
 	@ISA         = qw(Exporter);
-	@EXPORT      = qw(&SetArch &SetDistArch &GetArch &GetDistArch
+	@EXPORT      = qw(&CallRPM &SetArch &SetDistArch &GetArch &GetDistArch
 			&GetAllowedArch &GetAllowedDistArch &TestArch &IsElement
-			&SetLocalRPMs &AddLocalRPMs &CheckLocal &CompareLocal &SelectRPMs
-			&SetQueryRpmHeaders &SetQueryRpmDatabase &TieRpmDatabase);
+			&SetLocalRPMs &AddLocalRPMs &CheckLocal &CompareLocal &GetLocal
+			&SelectRPMs &LoadRpmBindings &TieRpmDatabase);
 	@EXPORT_OK   = qw();
 }
 END {};
 
-use vars qw($DEBUG $QueryRpmHeaders $QueryRpmDatabase);
+use vars qw($DEBUG $RpmModule $UsePipe $AddInstalled);
 
 $DEBUG = 0;
-$QueryRpmHeaders = 0;
-$QueryRpmDatabase = 0;
+$UsePipe = 0;
+$AddInstalled = 0;
+$RpmModule = 0;
+$RPM::err = "unknown error";
+
+my $RPM = "/bin/rpm";
+my $TmpFile = "autoupdate_$$.tmp";
+if ($ENV{'TMP'}) {
+	$TmpFile = "$ENV{'TMP'}/$TmpFile";
+} else {
+	$TmpFile = "/tmp/$TmpFile";
+}
+# Tags to query an rpm for
+my %TAGS = (
+	'NAME' => 0,
+	'VERSION' => 0,
+	'RELEASE' => 0,
+	'ARCH' => 0,
+	'EPOCH' => 0,
+	'VENDOR' => 0,
+	'PROVIDENAME' => 1,
+	'PROVIDEFLAGS' => 1,
+	'PROVIDEVERSION' => 1,
+	'REQUIRENAME' => 1,
+	'REQUIREFLAGS' => 1,
+	'REQUIREVERSION' => 1,
+	'OBSOLETENAME' => 1,
+	'OBSOLETEFLAGS' => 1,
+	'OBSOLETEVERSION' => 1 );
+# Build up the query format from %TAGS
+my $QueryFormat= "";
+foreach my $tag (keys %TAGS) {
+	if ($TAGS{$tag}) {
+		$QueryFormat .= "[$tag:%{$tag}\\n]";
+	} else {
+		$QueryFormat .= "$tag:%{$tag}\\n";
+	}
+}
+
 
 my $TieRpmDatabase = 0;
 my $Arch;
 my $DistArch;
+my $RPM_DB;
 my %LocalRPMs;
 my %LocalRPMsArchs;
 my @LocalTied;
+my @LocalMissing;
 my %RPM;
 
+######################
+# Calls RPM and saves all output
+######################
+
+sub CallRPM
+{
+	my $rpmoutput= shift;
+	# If a daemon gets restarted in the post script, the pipe
+	# doesn't get closed until the daemon exists under some
+	# older versions of rpm. 
+
+	if ($UsePipe) {
+		pipe(R, W);
+	}
+
+	my $pid = fork;
+	die("Could not fork for RPM: $!") unless defined($pid);
+	if(!$pid) {
+		if ($UsePipe) {
+			close(R);
+		} else {
+			# Just to be sure
+			if (-e $TmpFile) {
+				unlink($TmpFile) || die("Could not remove $TmpFile");
+			}
+			sysopen(W, $TmpFile, O_WRONLY|O_CREAT|O_EXCL, 00600) || die("Could not create: $TmpFile ($!)");
+		}
+		open(STDERR, ">&W") || die("Could not dup STDERR ($!)");
+		open(STDOUT, ">&W") || die("Could not dup STDOUT ($!)");
+		exec($RPM, @_) || die("Could not exec $RPM.");
+		close(W);
+		exit 1;
+	} else {
+		if ($UsePipe) {
+			close(W);
+		}	
+		waitpid($pid,0);
+		my $retval= $? >> 8;
+		unless ($UsePipe) {
+			open(R,"< $TmpFile") || die("Could not open: $TmpFile ($!)");
+		}
+		@{$rpmoutput}= <R>;
+		close(R);
+		unless ($UsePipe) {
+			unlink($TmpFile);
+		}
+		chomp(@{$rpmoutput});
+		return $retval;
+	}
+
+}
+
+######################
+# Query an rpm using CallRPM
+######################
+
+sub QueryRPM {
+	my @RPMOut;
+	my $flag = shift;
+	my $item = shift;
+	if ($flag eq "l") {
+		$flag = "-q";
+	} else {
+		$flag = "-qp"
+	}
+	unless ( CallRPM(\@RPMOut, $flag, "--queryformat", $QueryFormat, $item) == 0 ) {
+		$RPM::err= $RPMOut[0];
+		return undef;
+	}
+
+	my %HDR = ();
+	for my $line (@RPMOut) {
+		next unless ($line =~ /^([A-Z]+):(.*)$/);
+		my ($tag, $key) = ($1, $2);
+		next unless ( defined $TAGS{$tag} );
+		if ($TAGS{$tag}) {
+			if ( defined $HDR{$tag} ) {
+				push(@{$HDR{$tag}}, $key);
+			} else {
+				$HDR{$tag}= [ $key ];
+			}
+		} else {
+			$HDR{$tag} = $key;
+		}	
+	}
+	if ($HDR{'EPOCH'} and $HDR{'EPOCH'} eq '(none)') {
+		$HDR{'EPOCH'}= "";
+	}
+	return %HDR;
+}
+
 ##################################
 # Class for RPM header information
 ##################################
@@ -50,8 +179,9 @@ sub new {
 	my $name = "";
 	my $version = "";
 	my $release = "1";
-	my $epoch = "";
+	my $serial = "";
 	my $arch = "noarch";
+	my $vendor = "";
 	my $type = "r";
 	my $hdr = "";
 	return undef unless ($_[0]);
@@ -72,77 +202,53 @@ sub new {
 		$version= $2;
 		$release= $3;
 		$arch= $4;
-	} elsif ($file =~ /^(.*)-(\d[^-]*)-(\d[^-]*)\.rpm$/) {
-		$name= $1;
-		$version= $2;
-		$release= $3;
-	} elsif ($file =~ /^(.*)-(\d[^-]*)\.(\D[^.]+)\.rpm$/) {
-		$name= $1;
-		$version= $2;
-		$arch= $3;
-	} elsif ($file =~ /^(.*)-(\d[^-]*)\.rpm$/) {
-		$name= $1;
-		$version= $2;
 	}
 	if ($version =~ /^(\d+):(.+)$/) {
-		$epoch= $1;
+		$serial= $1;
 		$version=$2;
 	} elsif ($version =~ /^\(none\):(.+)$/) {
 		$version=$1;
 	}
 
-	return undef unless($name);
-	
 	$self->{NAME} = $name;
-	$self->{EPOCH} = $epoch;
+	$self->{EPOCH} = $serial;
 	$self->{VERSION} = $version;
 	$self->{RELEASE} = $release;
 	$self->{ARCH} = $arch;
+	$self->{VENDOR} = $vendor;
 	$self->{FILENAME} = "$path$file";
 	$self->{TYPE} = $type;
 	$self->{HDR} = $hdr;
 	bless ($self, $class);
-	return $self;
-}
 
-sub getheader {
-	my $self = shift;
-	my $hdr;
-
-	if ($self->{HDR}) {
-		return $self->{HDR};
-	}
-	if ($self->{TYPE} eq "r") {
-		return undef;
-	} elsif ($self->{TYPE} eq "l") {
-		return undef unless ($autoupdate::QueryRpmDatabase);
-		&autoupdate::TieRpmDatabase(1);
-		$hdr= $RPM{$self->name};
-		unless ($hdr)
-		{
-       			print STDERR "Error: " . $self->rpmname . ": $RPM::err\n";
-			$self->{TYPE} = "r";
-			return undef;
+	unless($name) {
+		if ($type eq "f") {
+			$hdr= $self->getheader();
+			if ($hdr) {
+				return $self;
+			} else {
+				return undef;
+			}
+		} elsif ($file =~ /^(.*)-(\d[^-]*)-(\d[^-]*)\.rpm$/) {
+			$name= $1;
+			$version= $2;
+			$release= $3;
+		} elsif ($file =~ /^(.*)-(\d[^-]*)\.(\D[^.]+)\.rpm$/) {
+			$name= $1;
+			$version= $2;
+			$arch= $3;
+		} elsif ($file =~ /^(.*)-(\d[^-]*)\.rpm$/) {
+			$name= $1;
+			$version= $2;
 		} else {
-			push(@LocalTied,$self);
-		}
-	} elsif ($self->{TYPE} eq "f") {
-		return undef unless ($autoupdate::QueryRpmHeaders);
-		$hdr = RPM::Header->new( $self->filename );
-		unless ($hdr)
-		{
-       		print STDERR "Error: " . $self->filename . ": $RPM::err\n";
-			$self->{TYPE} = "r";
 			return undef;
 		}
+		$self->{NAME} = $name;
+		$self->{VERSION} = $version;
+		$self->{RELEASE} = $release;
+		$self->{ARCH} = $arch;
 	}
-	$self->{NAME} = $hdr->{NAME};
-	$self->{EPOCH} = $hdr->{EPOCH};
-	$self->{VERSION} = $hdr->{VERSION};
-	$self->{RELEASE} = $hdr->{RELEASE};
-	$self->{ARCH} = $hdr->{ARCH};
-	$self->{HDR} = $hdr;
-	return $self->{HDR};
+	return $self;
 }
 
 sub name {
@@ -173,6 +279,161 @@ sub filename {
 	return $self->{FILENAME};
 }
 
+sub getheader {
+	my $self = shift;
+	my $hdr;
+
+	if ($self->{HDR}) {
+		return $self->{HDR};
+	}
+	if ($self->{TYPE} eq "r") {
+		return undef;
+	}
+
+	if ($self->{TYPE} eq "l") {
+		if ($autoupdate::RpmModule == 1) {
+			&autoupdate::TieRpmDatabase(1);
+			$hdr= $RPM{$self->name};
+		} elsif ($autoupdate::RpmModule == 2) {
+			&autoupdate::TieRpmDatabase(1);
+			my $i = $RPM_DB->find_by_name_iter($self->name);
+			unless ($i) {
+				$RPM::err= "Could not create package iterator.\n";
+				$hdr= undef;
+			} else {
+				while ($hdr= $i->next) {
+					last if ( $hdr->version eq $self->version and $hdr->release eq $self->release);				}
+			}
+		} else {
+			my %HDR = &autoupdate::QueryRPM("l", $self->name . "-" . $self->version);
+			$hdr= \%HDR;
+		}
+		unless ($hdr) {
+			print STDERR "Error: " . $self->rpmname . ": $RPM::err\n";
+			$self->{TYPE} = "r"; # Don't try again
+			return undef;
+		} else {
+			push(@LocalTied, $self);
+		}
+	} elsif ($self->{TYPE} eq "f") {
+		if ($autoupdate::RpmModule == 3) {
+			tie my %HDR, "RPM::Perlonly", $self->filename; 
+			$hdr = \%HDR;
+		} elsif ($autoupdate::RpmModule == 2) {
+			$RPM::err = "Failed to query\n";
+			eval { $hdr = RPM2->open_package( $self->filename ); }
+		} elsif ($autoupdate::RpmModule == 1) {
+			$hdr = RPM::Header->new( $self->filename );
+		} else {
+			my %HDR = &autoupdate::QueryRPM("f", $self->filename);
+			$hdr= \%HDR;
+		}
+		unless ($hdr) {
+       		print STDERR "Error: " . $self->filename . ": $RPM::err\n";
+			$self->{TYPE} = "r";
+			return undef;
+		}
+	}
+	$self->{HDR} = $hdr;
+	foreach my $tag ( qw(NAME EPOCH VERSION RELEASE ARCH VENDOR) ) {
+		$self->{$tag} = $self->gettag($tag);
+	}
+	return $hdr;
+}
+
+sub gettag {
+	my $self = shift;
+	my $tag = shift;
+	my $hdr= $self->{HDR};
+	unless ($hdr) {
+		$hdr= $self->getheader();
+		return undef unless ($hdr);
+	}
+	
+	if ($autoupdate::RpmModule == 2) {
+		$tag= lc($tag);
+		return $hdr->tag($tag);
+	} else {
+		$tag = uc($tag);
+		my $tmp = $hdr->{$tag};
+		return undef unless ($tmp);
+		if (ref($tmp) eq "ARRAY") {
+			return @{$tmp};
+		} else {
+			return $tmp;
+		}
+	}
+}
+
+sub files {
+	my $self = shift;
+	my ($flags, @files);
+
+	if ($autoupdate::RpmModule==0 or ($autoupdate::RpmModule==3 and $self->{TYPE} eq "l")) {
+		if ($self->{TYPE} eq "l") {
+			$flags= "-q";
+		} else {
+			$flags= "-qp";
+		}
+		unless ( &autoupdate::CallRPM(\@files, $flags, "--list", $self->filename) ==0 ) {
+			return ();
+		} else {
+			return @files;
+		}
+	}
+
+	my @base_names = $self->gettag('BASENAMES');
+	return () unless ($base_names[0]);
+	my @dir_names = $self->gettag('DIRNAMES');
+	my @dir_indexes = $self->gettag('DIRINDEXES');
+
+	foreach (0 .. $#base_names) {
+		push (@files, $dir_names[$dir_indexes[$_]] . $base_names[$_]);
+	}
+
+	return @files;
+}
+
+sub provides {
+	return decode_flags($_[0], "PROVIDE");
+}
+
+sub requires {
+	return decode_flags($_[0], "REQUIRE");
+}
+
+sub obsoletes {
+	return decode_flags($_[0], "OBSOLETE");
+}
+
+sub decode_flags {
+	my $self = shift;
+	my $item = shift;
+	my @ret= ();
+	my @name= $self->gettag("${item}NAME");
+	return () unless ($name[0]);
+	my @flag= $self->gettag("${item}FLAGS");
+	my @vers= $self->gettag("${item}VERSION");
+	for (my $i=0; $i< scalar(@name); $i++) {
+		my $ret = $name[$i];
+		next if (!$ret or $ret =~ /^rpmlib\(.*\)$/);
+		my $flag = $flag[$i];
+		if ($flag) {
+			$flag = $flag & 14;
+		}
+		if ($flag) {
+			$ret .= " ";
+			$ret .= "<" if ($flag & 2);
+			$ret .= ">" if ($flag & 4);
+			$ret .= "=" if ($flag & 8);
+			$ret .= " " . $vers[$i];
+		}
+		push(@ret, $ret) unless &autoupdate::IsElement($ret, @ret);
+	}
+
+	return @ret;
+}
+
 #
 # Version comparison code
 #
@@ -187,22 +448,39 @@ sub compare {
 	my $b = $rpmb->{NAME};
 	if ($a ne $b) { return ($a cmp $b); }
 	if ($rpma->version eq $rpmb->version) { return 0; }
-	if ($autoupdate::QueryRpmHeaders and $rpma and $rpmb) {
+	if ($rpma and $rpmb and ($autoupdate::RpmModule==1 or $autoupdate::RpmModule==2)) {
 		$a = $rpma->getheader();
 		$b = $rpmb->getheader();
 		if ($a and $b) {
 			#Compare using rpm
-			my $tmp= $a->cmpver($b);
+			my $tmp;
+			if ($autoupdate::RpmModule == 2) {
+				$RPM::err = "Failed to compare\n";
+				$tmp= $a->compare($b);
+			} else {
+				$tmp= $a->cmpver($b);
+			}
 			unless (defined($tmp)) {
 				print STDERR "Error: $RPM::err\n";
 			}
+			if ($tmp < 0) {
+				$tmp = -1;
+			} elsif ($tmp > 0) {
+				$tmp = 1;
+			}
 			return $tmp;
 		}
 	}
+	if ($rpma->{TYPE} eq "f" and !$rpma->{HDR}) {
+		$rpma->getheader();
+	}
+	if ($rpmb->{TYPE} eq "f" and !$rpmb->{HDR}) {
+		$rpmb->getheader();
+	}
 
 	$a = $rpma->{EPOCH};
 	$b = $rpmb->{EPOCH};
-	if ($a and $b) { return ($a <=> $b)};
+	if ($a and $b and $a ne $b) { return ($a <=> $b)};
 
 	$a = $rpma->{VERSION};
 	$b = $rpmb->{VERSION};
@@ -226,6 +504,13 @@ sub compare {
 			$b=$rb;
 		}
 	}
+	# Rename pre1.0 -> 1.0pre
+	if ( $a =~ /^pre(.+)$/) {
+		$a= $1 . "pre";
+	}
+	if ( $b =~ /^pre(.+)$/) {
+		$b= $1 . "pre";
+	}
 
 	# Split off equal parts first
 	my ($a1,$b1);
@@ -354,7 +639,7 @@ sub SetArch {
 			if ( open(FILE,"</proc/cpuinfo") ) {
 				my $item;
 				foreach $item (<FILE>) {
-					next unless $item =~ /cpu model.*ev6/i;
+					next unless ($item =~ /cpu model.*ev6/i);
 					$Arch="alphaev6";
 					last;
 				}
@@ -364,7 +649,7 @@ sub SetArch {
 			if ( open(FILE,"</proc/cpuinfo") ) {
 				my $item;
 				foreach $item (<FILE>) {
-					next unless $item =~ /model name.*athlon/i;
+					next unless ($item =~ /model name.*(athlon|duron)/i) ;
 					$Arch="athlon";
 					last;
 				}
@@ -476,14 +761,21 @@ sub IsElement
 
 sub SetLocalRPMs
 {
+	print "AUPM: Resetting local rpms.\n" if $DEBUG;
 	%LocalRPMs= ();
 	%LocalRPMsArchs= ();
-	SetQueryRpmDatabase(0);
-	AddLocalRPMs( @_ ) if (@_);
+	TieRpmDatabase(0);
+	if ($_[0] and $_[0] eq "l") {
+		print "AUPM: Adding installed rpms.\n" if $DEBUG;
+		TieRpmDatabase(1);
+		$AddInstalled= 1;
+	} else {
+		$AddInstalled= 0;
+	}
 }
 
 ######################
-# Check if a local rpm exists
+# Add a local rpm exists
 ######################
 
 sub AddLocalRPMs
@@ -495,21 +787,18 @@ sub AddLocalRPMs
 	}
 	foreach $item ( @_ ) {
 		$tmp = Header->new($item,$type);
-		$name = $tmp->name;
 		next unless $tmp;
+		$name = $tmp->name;
 		if ( CheckLocal($name) ) {
 			$loc= $LocalRPMs{$name};
 			$res= $tmp->compare($loc);
-			if ($res==1) {
-				$LocalRPMs{$name}= $tmp;
-				$LocalRPMsArchs{$name}= [ $tmp->arch ];
-			} elsif ($res ==0 and ($tmp->version eq $loc->version)) {
+			if ($res ==0 and ($tmp->version eq $loc->version)) {
 				push(@{$LocalRPMsArchs{$name}}, $tmp->arch); 
 			}
-		} else {
-			$LocalRPMs{$name}=$tmp;
-			$LocalRPMsArchs{$name}= [ $tmp->arch ];
+			next if ($res < 1);
 		}
+		$LocalRPMs{$name}= $tmp;
+		$LocalRPMsArchs{$name}= [ $tmp->arch ];
 	}
 }
 
@@ -521,21 +810,25 @@ sub CheckLocal
 {
 	my $name= shift;
 	my $hdr= $LocalRPMs{$name};
-	if ( $hdr ) {
-		return $hdr->version;
-	} elsif ($QueryRpmDatabase) {
-		TieRpmDatabase(1);
-		$hdr= $RPM{$name};
-		if ($hdr) {
-			my $tmp= Header->new($hdr->{NAME} . "-" . $hdr->{VERSION} . "-" . $hdr->{RELEASE} . "." . $hdr->{ARCH} . ".rpm","l");
-			$tmp->{EPOCH}= $hdr->{EPOCH};
-			$tmp->{HDR}= $hdr;
-			$LocalRPMs{$name}= $tmp;
-			push(@LocalTied,$tmp);
-			return $tmp->version;
-		}
+	if ($AddInstalled and !defined $hdr) {
+		$hdr= GetInstalled($name);
 	}
-	return 0;
+	return 0 unless ($hdr);
+	return $hdr->version;
+}
+
+######################
+# Get the header of a local rpm
+######################
+
+sub GetLocal
+{
+	my $name= shift;
+	my $hdr= $LocalRPMs{$name};
+	if ($AddInstalled and !defined $hdr) {
+		$hdr= GetInstalled($name);
+	}
+	return $hdr;
 }
 
 ######################
@@ -544,59 +837,50 @@ sub CheckLocal
 
 sub CompareLocal
 {
-	my $name= $_[0]->name;
-	my $hdr= $LocalRPMs{$name};
-	unless ( $hdr or !$QueryRpmDatabase) {
-		TieRpmDatabase(1);
-		$hdr= $RPM{$name};
-                if ($hdr) {
-                        my $tmp= Header->new($hdr->{NAME} . "-" . $hdr->{VERSION} . "-" . $hdr->{RELEASE} . "." . $hdr->{ARCH} . ".rpm","l");
-                        $tmp->{EPOCH}= $hdr->{EPOCH};
-                        $tmp->{HDR}= $hdr;
-                        $LocalRPMs{$name}= $tmp;
-                        push(@LocalTied,$tmp);
-                        $hdr=$tmp;
-                }
-	}
+	my $hdr= GetLocal($_[0]->name);
 	return 1 unless ($hdr);
 	return $_[0]->compare($hdr);
 }
 
 ######################
-# Should we try to query rpm headers
+# Load the preferred perl module for rpm
 ######################
 
-sub SetQueryRpmHeaders {
-	if (defined($_[0]) and ($_[0] ne $QueryRpmHeaders) ) {
-		$QueryRpmHeaders= $_[0];
-		if ($QueryRpmHeaders) {
-			eval "use RPM::Header";
-			if ($@) { 
-				warn('Failed to load RPM::Header');
-				$QueryRpmHeaders= 0;
-			}
-		}
+sub LoadRpmBindings {
+	if ($RpmModule) {
+		return $RpmModule;
 	}
-	return $QueryRpmHeaders;
-}
+	$RpmModule = $_[0];
 
-######################
-# Should we try to query the rpm db
-######################
-
-sub SetQueryRpmDatabase {
-	if (defined($_[0]) and ($_[0] ne $QueryRpmDatabase)) {
-		$QueryRpmDatabase= $_[0];
-		if ($QueryRpmDatabase) {
-			eval "use RPM::Header";
-			if ($@) { 
-				warn('Failed to load RPM::Header');
-				$QueryRpmDatabase= 0;
-			}
-		}
-		TieRpmDatabase($QueryRpmDatabase);
+	my @modules;
+	if ($RpmModule eq "0") {
+		return $RpmModule;
+	} elsif ($RpmModule eq "1") {
+		@modules = qw(RPM::Header);
+	} elsif ($RpmModule eq "2") {
+		@modules = qw(RPM2);
+	} elsif ($RpmModule eq "3") {
+		@modules = qw(RPM::Perlonly);
+	} else {
+		$RpmModule= 1;
+		@modules = qw(RPM::Header RPM2 RPM::Perlonly);
 	}
-	return $QueryRpmDatabase;
+	foreach my $module (@modules) {
+		print "AUPM: Loading $module module: " if $DEBUG;
+		eval "use $module";
+		unless ($@) {
+			print "ok\n" if $DEBUG;
+			last;
+		}
+		print "failed\n" if $DEBUG;
+		$RpmModule++;
+	}
+	if ($@) { 
+		warn('Failed to load RPM Module') if ($_[0]);
+		$RpmModule= 0;
+	}
+
+	return $RpmModule;
 }
 
 ######################
@@ -607,26 +891,106 @@ sub TieRpmDatabase {
 	return 0 if ($_[0] eq $TieRpmDatabase);
 	$TieRpmDatabase=0;
 	unless ($_[0]) {
-		print "Closing rpm db.\n" if $DEBUG;
+		print "AUPM: Closing rpm db.\n" if $DEBUG;
 		foreach my $item (@LocalTied) {
 			$item->{HDR}= "";
 			if ($LocalRPMs{$item->name} and $LocalRPMs{$item->name} eq $item) {
 				delete $LocalRPMs{$item->name};
 			}
 		}
+		foreach my $item (@LocalMissing) {
+			if ($LocalRPMs{$item} and $LocalRPMs{$item} ne "") {
+				delete $LocalRPMs{$item};
+			}
+		}
 		@LocalTied= ();
-		untie(%RPM);
+		@LocalMissing= ();
+		if ($RpmModule == 1) {
+			untie(%RPM);
+		} elsif ($RpmModule == 2) {
+			$RPM_DB->close_rpm_db();
+			$RPM_DB= undef;
+		}
 		return 0;
 	}
-	print "Opening rpm db.\n" if $DEBUG;
-	unless ( tie(%RPM, "RPM::Database") ) {
-		print STDERR "Fatal: Could not tie rpm database.\n";
-		die ($RPM::err);
+	print "AUPM: Opening rpm db.\n" if $DEBUG;
+	if ($RpmModule == 1) {
+		unless ( tie(%RPM, "RPM::Database") ) {
+			print STDERR "Fatal: Could not tie rpm database.\n";
+			die ($RPM::err);
+		}		
+	} elsif ($RpmModule == 2) {
+		unless ( $RPM_DB = RPM2->open_rpm_db() ) {
+			die("Fatal: Could not tie rpm database.");
+		}
 	}
 	$TieRpmDatabase=1;
 	return 0;
 }
 
+######################
+# Get a header from the rpm database
+######################
+
+sub GetInstalled {
+	my $hdr;
+	my $name = shift;
+
+	&TieRpmDatabase(1);
+	if ($RpmModule == 2) {
+		my $i = $RPM_DB->find_by_name_iter($name);
+		unless ($i) {
+			$RPM::err= "Could not create package iterator.\n";
+			return undef;
+		}
+		while (my $tmp = $i->next) {
+			if (!$hdr or $tmp->compare($hdr) == 1) {
+				$hdr= $tmp;
+			}
+		}
+	} elsif ($RpmModule == 1) {
+		$hdr= $RPM{$name};
+	} else {
+		my @RPMOut;
+		if ( CallRPM(\@RPMOut, "-q", "--queryformat", "%{NAME}-%{EPOCH}:%{VERSION}-%{RELEASE}.%{ARCH}.rpm\n", $name) == 0 ) {
+			foreach my $item ( @RPMOut ) {
+				my $tmp = Header->new($item, "l");
+				next unless $tmp;
+				$name = $tmp->name;
+				next if ($tmp->compare($LocalRPMs{$name}) < 1);
+				$LocalRPMs{$name}= $tmp;
+				$LocalRPMsArchs{$name}= [ $tmp->arch ];
+			}
+			return $LocalRPMs{$name};
+		} else {
+			# Not installed
+			$hdr= undef;
+		}
+	}
+	unless ($hdr) {
+		$RPM::err= "Local header not found.\n";
+		$LocalRPMs{$name}= ""; # Don't try again
+		push(@LocalMissing, $name);
+		return undef;
+	}
+	my $tmp;
+	if ($RpmModule == 2) {
+		$tmp= $hdr->name . "-" . $hdr->version . "-" . $hdr->release . "." . $hdr->arch . ".rpm";
+	} else {
+		$tmp= $hdr->{NAME} . "-" . $hdr->{VERSION} . "-" . $hdr->{RELEASE} . "." . $hdr->{ARCH} . ".rpm";
+	}
+	$tmp= Header->new($tmp, "l");
+	$tmp->{HDR}= $hdr;
+	$tmp->{EPOCH}= $tmp->gettag("EPOCH");
+	$tmp->{VENDOR}= $tmp->gettag("VENDOR");
+	$LocalRPMs{$name}= $tmp;
+	$LocalRPMsArchs{$name}= [ $tmp->arch ];
+	push(@LocalTied, $tmp);
+	$hdr=$tmp;
+
+	return $hdr;
+}
+
 ######################
 # Select rpms from a given array of rpms
 # Returns array of selected rpms 
@@ -677,8 +1041,11 @@ sub SelectRPMs
 			print "(wrong arch)\n" if $DEBUG;
 			next;
 		}
-		$name=$rpm->name;		
-		if ( $CheckLocal ) { # Check against local version
+		$name=$rpm->name;
+	
+		if ($Version{$name}) {
+			$tmp = $rpm->compare($Version{$name}->[0]);
+		} elsif ( $CheckLocal ) { # Check against local version
 			unless ( CheckLocal($name) or $CheckLocal == 2 ) {
 				print "(no local version)\n" if $DEBUG;
 				next;
@@ -688,15 +1055,13 @@ sub SelectRPMs
 			if ( $res == -1 or ($res == 0 and ($BestMatch or ($rpm->version ne $loc->version) )) ) {
 				print "(have local version)\n" if $DEBUG;
 				next;
-			} elsif ($res ==0 and IsElement($rpm->arch, @{$LocalRPMsArchs{$name}})) {
+			} elsif ($res ==0 and ($rpm->arch eq $loc->arch or IsElement($rpm->arch, @{$LocalRPMsArchs{$name}}))) {
 				print "(have local version)\n" if $DEBUG;
 				next;
 			}
-		}
-		if ($Version{$name}) {
-			$tmp = $rpm->compare($Version{$name}->[0]);
+			$tmp =1;
 		} else {
-			$tmp = 1;
+			$tmp =1;
 		}
 
 		if ( $tmp == 1 ) { # Newer then what we currently have
@@ -787,8 +1152,8 @@ autoupdate.pm
 
  &SetArch &SetDistArch &TestArch
  &GetArch &GetDistArch &GetAllowedArch &GetAllowedDistArch
- &SetLocalRPMs &AddLocalRPMs &CheckLocal &CompareLocal &SelectRPMs
- &SetQueryRpmHeaders &SetQueryRpmDatabase &TieRpmDatabase
+ &SetLocalRPMs &AddLocalRPMs &CheckLocal &CompareLocal &GetLocal
+ &SelectRPMs &LoadRpmBindings &TieRpmDatabase
  &IsElement
 
 =head1 DESCRIPTION
@@ -811,7 +1176,7 @@ be used to specify the type.
  "r(emote)" the rpm corresponds to a remote file which cannot
          be queried
 
-The string shouold be of the form "[/path/]name-[epoch:]version-release.arch.rpm".
+The string shouold be of the form "[/path/]name-[serial:]version-release.arch.rpm".
 
 Methods implemented are
 
@@ -823,6 +1188,14 @@ Methods implemented are
 
  $rpm1->version($rpm2) compares the versions of $rpm1 and $rpm2.
 
+ The remaining methods only work if rpm bindings are used
+
+ $rpm->provides  like 'rpm -q --requires'
+ $rpm->obsoletes like 'rpm -q --obsoletes'
+ $rpm->requires  like 'rpm -q --requires'
+ $rpm->files     like 'rpm -ql'
+ $rpm->gettag('TAG') retrieve a tag from an rpm header
+
 Each object contains the following date:
 
  $rpm->{NAME}     name tag of the rpm
@@ -838,6 +1211,9 @@ Each object contains the following date:
 
 First of all there the following functions related to architectures defined:
 
+ &CallRPM(\@Out, @Args)    Calls rpm with the given arguments and saves the output
+                           in @Out. Returns the exit code of rpm.
+
  &SetArch([$string])       Sets the architecture to the given value or (if none is
                            given) tries to determine it from the current machine.
  &SetDistArch([$string])   Same for distribution architecture
@@ -847,12 +1223,15 @@ First of all there the following functions related to architectures defined:
  &GetAllowedDistArch       Returns an array of allowed architectures
  &TestArch                 Checks if the arhitecture is known
 
- &SetLocalRPMs(@rpms)      Takes an array of file names and builds a hash of local                                      rpms
+ &SetLocalRPMs([l])        Resets the hash of local rpms. Adds all installed rpms if
+                           the first argument is "l".                           
  &AddLocalRPMs(@rpms)      Adds file names to the hash of local rpms
  &CheckLocal($name)        Checks if an rpm is in the hash of local rpms and returns
                            its version
  &CompareLocal($rpm)       Compares an rpm against the local version and returns the
                            result
+ &GetLocal($name)          Returns the header of the newest local rpm with
+                           the given name
 
  &SelectRPMs(\@rpms, 0|1, 0|1|2) Selects rpms from the given array by comparing them
                            to the local hash. Only the rpm with the highest version
@@ -862,9 +1241,8 @@ First of all there the following functions related to architectures defined:
                            are selected, if it is 2 only rpms which are newer than the
                            local one or for which no local version exists are selected.
 
- &SetQueryRpmHeaders(0|1)  Should rpms (of type file) be queried using the rpm                                          bindings.
- &SetQueryRpmDatabase(0|1) Should the local rpm data base be queried using the rpm
-                           bindings.
+ &LoadRpmBindings(0|1|2|3) Load the preferred RPM bindings:
+                           0 = automatic, 1 = RPM, 2 = RPM2, 3= RPM::Perlonly 
  &TieRpmDatabase(0|1)      Tie, untie the local rpm data base.
  
  &IsElement($item,@array)  Tests if $item is element of @array.
